from redis import Redis


class RedisGears(Redis):
    def abortexecution(self):
        """
        docstring
        """
        raise NotImplementedError

    def configget(self):
        """
        docstring
        """
        raise NotImplementedError

    def configset(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def dropexecution(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def dumpexecutions(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def dumpregistrations(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def getexecution(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def getresults(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def getresultsblocking(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def infocluster(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def pyexecute(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def pstats(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def pydumpreqs(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def refreshcluster(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def trigger(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError

    def unregister(self, parameter_list):
        """
        docstring
        """
        raise NotImplementedError



from enum import Enum
import logging

# Various Helper enums, instead of "magic" strings.


class StrEnum(Enum):
    """
    Enumeration for which its string representatin is the enumeration value
    """
    def __str__(self):
        return str(self.value)

    def __eq__(self, other):
        return str(self) == str(other)


class Reader(StrEnum):
    """Redis Gears Reader Types"""
    KeysReader = "KeysReader"
    KeysOnlyReader = "KeysOnlyReader"
    StreamReader = "StreamReader"
    PythonReader = "PythonReader"
    ShardsIDReader = "ShardsIDReader"
    CommandReader = "CommandReader"


class TriggerMode(StrEnum):
    """Redis Geears Trigger modes for registered actions"""
    Async = 'async'
    AsyncLocal = 'async_local'
    Sync = 'sync'


class LogLevel(StrEnum):
    """Redis Gears log levels"""
    Debug = 'debug'
    Verbose = 'verbose'
    Notice = 'notice'
    Warning = 'warninig'

    def __int__(self):
        if self == LogLevel.Debug:
            return logging.DEBUG
        if self == LogLevel.Verbose:
            return logging.INFO
        if self == LogLevel.Notice:
            return logging.WARNING
        if self == LogLevel.Warning:
            return logging.ERROR  # Maybe a little unintuitive ;)



import sys


if 'redisgears' not in sys.modules:
    from .placeholders import (   # noqa: F401
        atomic, execute, hashtag, log,
        gearsConfigGet, configGet,
        GearsBuilder, GB,
    )
else:
    atomic
    execute
    hashtag
    log
    gearsConfigGet
    configGet
    GearsBuilder
    GB

    # from redisgears import atomicCtx as atomic  # noqa: F401
    # from redisgears import executeCommand as execute  # noqa: F401
    # from redisgears import getMyHashTag as hashtag  # noqa: F401
    # from redisgears import log  # noqa: F401
    # from redisgears import config_get as configGet  # noqa: F401

    # def gearsConfigGet(key, default=None):
    #     val = configGet(key)
    #     return val if val is not None else default

    # from redisgears import callNext as call_next
    # from redisgears import registerTimeEvent as registerTE
    # from redisgears import gearsCtx
    # from redisgears import gearsFutureCtx as gearsFuture
    # from redisgears import setGearsSession
    # from redisgears import getGearsSession
    # from redisgears import registerGearsThread
    # from redisgears import isInAtomicBlock
